@@ -41,7 +41,7 @@ from traceback import format_exception
 
 import urllib.parse
 
-from . import s_log , s_config , s_net , s_db
+from . import get_version , s_log , s_config , s_net , s_db
 
 #####################################################################
 
@@ -74,8 +74,8 @@ def json_save( fn , data ) :
     #with open( fn , 'w', encoding='utf-8') as f:
     #    json.dump(data, f, ensure_ascii=False, indent=4)
 
-    with open( fn , "w" ) as file:
-        file.write( json_encode( data) )
+    with open( fn , "w" ) as file :
+        file.write( json_encode( data ) )
 
 def json_load( fn ) :
     with open( fn ) as f :
@@ -108,6 +108,10 @@ def json_prettyprint( data ) :
 
 #####################################################################
 
+
+def get_ver( ) :
+    return( get_version( ) )
+
 def num_randomlist( s = 10 ) :
     random_list = [random.random() for i in range(s)]
     return(random_list)
@@ -207,7 +211,10 @@ def save_hashkeycontent( fk , fc ) :
 
     path_mkdirs( fd )
 
-    fp = fd + hash( fk ) + ".session"
+    hk = fk + s_config.get_key( "endpoint/token" )
+
+    fp = fd + hash( hk ) + ".session"
+
     with open( fp , "w" ) as file :
         file.write( fc )
 
@@ -233,6 +240,7 @@ def hash( str , prefix = "" ) :
 def hash_file_md5( fp ) :
     return( hashlib.md5( open( fp , "rb" ).read( ) ).hexdigest( ) )
 
+# FIXME TODO add encode...??? especially images...
 def base64_decode( str ) :
     m = base64.b64decode( str ).decode( "utf-8" )
     return( m )
@@ -504,7 +512,29 @@ def container_appenvstatic_get( ) :
     if "_CONTAINER_LOCAL_APPENVSTATIC" in os.environ :
         ENV_STATIC = dict( urllib.parse.parse_qsl( os.environ[ "_CONTAINER_LOCAL_APPENVSTATIC" ] ) )
     return( ENV_STATIC )
-   
+  
+
+def display_time( seconds , granularity = 2 ) :
+
+    intervals = (
+        ( 'weeks' , 604800 ) , 
+        ( 'days' , 86400 ) ,  
+        ( 'hours' , 3600 ) , 
+        ( 'minutes' , 60 ) ,
+        ( 'seconds' , 1 ) ,
+    )
+
+    result = [ ]
+
+    for name, count in intervals :
+        value = seconds // count
+        if value :
+            seconds -= value * count
+            if value == 1 :
+                name = name.rstrip( 's' )
+            result.append( "{} {}".format( value , name ) )
+    return ', '.join( result[ :granularity ] )
+
 
 
 def datetime_iso( ) :
@@ -535,6 +565,13 @@ def idle( ) :
         s_log.write( "IDLE" )
         time.sleep( 86400 )
 
+
+def datetime_nowutcstr( td = 0 ) :
+
+    d = datetime.utcnow( ) + timedelta( seconds = td)
+
+    return( d.strftime( "%Y-%m-%dT%H:%M:%S" ) )
+
 def datetime_now_db( td = 0 ) :
 
     d = datetime.now( ) + timedelta( seconds = td)
